@@ -48,9 +48,8 @@
 #define SAP_SERVER_INTERFACE	"org.bluez.SimAccess"
 #define SAP_UUID		"0000112D-0000-1000-8000-00805F9B34FB"
 #define SAP_SERVER_CHANNEL	8
-#define SAP_BUF_SIZE		512
 
-#define PADDING4(x) (4 - (x & 0x03))
+#define PADDING4(x) ((4 - ((x) & 0x03)) & 0x03)
 #define PARAMETER_SIZE(x) (sizeof(struct sap_parameter) + x + PADDING4(x))
 
 #define SAP_NO_REQ 0xFF
@@ -61,6 +60,7 @@
 enum {
 	SAP_STATE_DISCONNECTED,
 	SAP_STATE_CONNECT_IN_PROGRESS,
+	SAP_STATE_CONNECT_MODEM_BUSY,
 	SAP_STATE_CONNECTED,
 	SAP_STATE_GRACEFUL_DISCONNECT,
 	SAP_STATE_IMMEDIATE_DISCONNECT,
@@ -254,22 +254,24 @@ static int send_message(struct sap_connection *conn, void *buf, size_t size)
 	if (!conn || !buf)
 		return -EINVAL;
 
-	DBG("size %zu", size);
+	DBG("conn %p, size %zu", conn, size);
 
 	gstatus = g_io_channel_write_chars(conn->io, buf, size, &written,
-						&gerr);
+									&gerr);
 	if (gstatus != G_IO_STATUS_NORMAL) {
 		if (gerr)
 			g_error_free(gerr);
 
 		error("write error (0x%02x).", gstatus);
-		return -EINVAL;
+		return -EIO;
 	}
 
-	if (written != size)
-		error("write error.(written %zu size %zu)", written, size);
+	if (written != size) {
+		error("written %zu bytes out of %zu", written, size);
+		return -EIO;
+	}
 
-	return 0;
+	return written;
 }
 
 static int disconnect_ind(void *sap_device, uint8_t disc_type)
@@ -338,8 +340,6 @@ static void connect_req(struct sap_connection *conn,
 	return;
 
 error_rsp:
-	error("Processing error (param %p state %d pr 0x%02x)", param,
-					conn->state, conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -349,11 +349,10 @@ static int disconnect_req(struct sap_connection *conn, uint8_t disc_type)
 
 	switch (disc_type) {
 	case SAP_DISCONNECTION_TYPE_GRACEFUL:
-		if (conn->state == SAP_STATE_DISCONNECTED)
-			goto error_req;
-
-		if (conn->state == SAP_STATE_CONNECT_IN_PROGRESS)
-			goto error_req;
+		if (conn->state == SAP_STATE_DISCONNECTED ||
+				conn->state == SAP_STATE_CONNECT_IN_PROGRESS ||
+				conn->state == SAP_STATE_CONNECT_MODEM_BUSY)
+			return -EPERM;
 
 		if (conn->state == SAP_STATE_CONNECTED) {
 			conn->state = SAP_STATE_GRACEFUL_DISCONNECT;
@@ -367,11 +366,10 @@ static int disconnect_req(struct sap_connection *conn, uint8_t disc_type)
 		return 0;
 
 	case SAP_DISCONNECTION_TYPE_IMMEDIATE:
-		if (conn->state == SAP_STATE_DISCONNECTED)
-			goto error_req;
-
-		if (conn->state == SAP_STATE_CONNECT_IN_PROGRESS)
-			goto error_req;
+		if (conn->state == SAP_STATE_DISCONNECTED ||
+				conn->state == SAP_STATE_CONNECT_IN_PROGRESS ||
+				conn->state == SAP_STATE_CONNECT_MODEM_BUSY)
+			return -EPERM;
 
 		if (conn->state == SAP_STATE_CONNECTED ||
 				conn->state == SAP_STATE_GRACEFUL_DISCONNECT) {
@@ -387,8 +385,10 @@ static int disconnect_req(struct sap_connection *conn, uint8_t disc_type)
 
 	case SAP_DISCONNECTION_TYPE_CLIENT:
 		if (conn->state != SAP_STATE_CONNECTED &&
-				conn->state != SAP_STATE_GRACEFUL_DISCONNECT)
-			goto error_rsp;
+				conn->state != SAP_STATE_GRACEFUL_DISCONNECT) {
+			sap_error_rsp(conn);
+			return -EPERM;
+		}
 
 		conn->state = SAP_STATE_CLIENT_DISCONNECT;
 		conn->processing_req = SAP_NO_REQ;
@@ -402,13 +402,6 @@ static int disconnect_req(struct sap_connection *conn, uint8_t disc_type)
 		error("Unknown disconnection type (0x%02x).", disc_type);
 		return -EINVAL;
 	}
-
-error_rsp:
-	sap_error_rsp(conn);
-error_req:
-	error("Processing error (state %d pr 0x%02x)", conn->state,
-						conn->processing_req);
-	return -EPERM;
 }
 
 static void transfer_apdu_req(struct sap_connection *conn,
@@ -434,8 +427,6 @@ static void transfer_apdu_req(struct sap_connection *conn,
 	return;
 
 error_rsp:
-	error("Processing error (param %p state %d pr 0x%02x)", param,
-					conn->state, conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -455,8 +446,6 @@ static void transfer_atr_req(struct sap_connection *conn)
 	return;
 
 error_rsp:
-	error("Processing error (state %d pr 0x%02x)", conn->state,
-						conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -476,8 +465,6 @@ static void power_sim_off_req(struct sap_connection *conn)
 	return;
 
 error_rsp:
-	error("Processing error (state %d pr 0x%02x)", conn->state,
-						conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -497,8 +484,6 @@ static void power_sim_on_req(struct sap_connection *conn)
 	return;
 
 error_rsp:
-	error("Processing error (state %d pr 0x%02x)", conn->state,
-						conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -518,8 +503,6 @@ static void reset_sim_req(struct sap_connection *conn)
 	return;
 
 error_rsp:
-	error("Processing error (state %d pr 0x%02x param)", conn->state,
-						conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -539,8 +522,6 @@ static void transfer_card_reader_status_req(struct sap_connection *conn)
 	return;
 
 error_rsp:
-	error("Processing error (state %d pr 0x%02x)", conn->state,
-						conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -564,8 +545,6 @@ static void set_transport_protocol_req(struct sap_connection *conn,
 	return;
 
 error_rsp:
-	error("Processing error (param %p state %d pr 0x%02x)", param,
-					conn->state, conn->processing_req);
 	sap_error_rsp(conn);
 }
 
@@ -626,6 +605,17 @@ static gboolean guard_timeout(gpointer data)
 	return FALSE;
 }
 
+static void sap_set_connected(struct sap_connection *conn)
+{
+	gboolean connected = TRUE;
+
+	emit_property_changed(connection, server->path,
+					SAP_SERVER_INTERFACE,
+		"Connected", DBUS_TYPE_BOOLEAN, &connected);
+
+	conn->state = SAP_STATE_CONNECTED;
+}
+
 int sap_connect_rsp(void *sap_device, uint8_t status, uint16_t maxmsgsize)
 {
 	struct sap_connection *conn = sap_device;
@@ -654,8 +644,7 @@ int sap_connect_rsp(void *sap_device, uint8_t status, uint16_t maxmsgsize)
 	size += PARAMETER_SIZE(SAP_PARAM_ID_CONN_STATUS_LEN);
 
 	/* Add MaxMsgSize */
-	if (maxmsgsize && (status == SAP_STATUS_MAX_MSG_SIZE_NOT_SUPPORTED ||
-				status == SAP_STATUS_MAX_MSG_SIZE_TOO_SMALL)) {
+	if (maxmsgsize) {
 		uint16_t *len;
 
 		msg->nparam++;
@@ -668,13 +657,10 @@ int sap_connect_rsp(void *sap_device, uint8_t status, uint16_t maxmsgsize)
 	}
 
 	if (status == SAP_STATUS_OK) {
-		gboolean connected = TRUE;
-
-		emit_property_changed(connection, server->path,
-						SAP_SERVER_INTERFACE,
-			"Connected", DBUS_TYPE_BOOLEAN, &connected);
-
-		conn->state = SAP_STATE_CONNECTED;
+		sap_set_connected(conn);
+	} else if (status == SAP_STATUS_OK_ONGOING_CALL) {
+		DBG("ongoing call. Wait for reset indication!");
+		conn->state = SAP_STATE_CONNECT_MODEM_BUSY;
 	} else {
 		conn->state = SAP_STATE_DISCONNECTED;
 
@@ -967,6 +953,9 @@ int sap_error_rsp(void *sap_device)
 	memset(&msg, 0, sizeof(msg));
 	msg.id = SAP_ERROR_RESP;
 
+	error("SAP error (state %d pr 0x%02x).", conn->state,
+							conn->processing_req);
+
 	return send_message(conn, &msg, sizeof(msg));
 }
 
@@ -984,6 +973,11 @@ int sap_status_ind(void *sap_device, uint8_t status_change)
 	DBG("state %d pr 0x%02x sc 0x%02x", conn->state, conn->processing_req,
 				status_change);
 
+	/* Might be need to change state to connected after ongoing call.*/
+	if (conn->state == SAP_STATE_CONNECT_MODEM_BUSY &&
+			status_change == SAP_STATUS_CHANGE_CARD_RESET)
+		sap_set_connected(conn);
+
 	if (conn->state != SAP_STATE_CONNECTED &&
 			conn->state != SAP_STATE_GRACEFUL_DISCONNECT)
 		return 0;
@@ -1008,10 +1002,9 @@ int sap_disconnect_ind(void *sap_device, uint8_t disc_type)
 	return disconnect_req(conn, SAP_DISCONNECTION_TYPE_IMMEDIATE);
 }
 
-static int handle_cmd(void *data, void *buf, size_t size)
+static int handle_cmd(struct sap_connection *conn, void *buf, size_t size)
 {
 	struct sap_message *msg = buf;
-	struct sap_connection *conn = data;
 
 	if (!conn)
 		return -EINVAL;
@@ -1055,12 +1048,12 @@ static int handle_cmd(void *data, void *buf, size_t size)
 		set_transport_protocol_req(conn, msg->param);
 		return 0;
 	default:
-		DBG("SAP unknown message.");
+		DBG("Unknown SAP message id 0x%02x.", msg->id);
 		break;
 	}
 
 error_rsp:
-	DBG("Bad request message format.");
+	DBG("Invalid SAP message format.");
 	sap_error_rsp(conn);
 	return -EBADMSG;
 }
@@ -1084,12 +1077,14 @@ static void sap_conn_remove(struct sap_connection *conn)
 
 static gboolean sap_io_cb(GIOChannel *io, GIOCondition cond, gpointer data)
 {
+	struct sap_connection *conn = data;
+
 	char buf[SAP_BUF_SIZE];
 	size_t bytes_read = 0;
 	GError *gerr = NULL;
 	GIOStatus gstatus;
 
-	DBG("io %p", io);
+	DBG("conn %p io %p", conn, io);
 
 	if (cond & G_IO_NVAL) {
 		DBG("ERR (G_IO_NVAL) on rfcomm socket.");
@@ -1107,7 +1102,7 @@ static gboolean sap_io_cb(GIOChannel *io, GIOCondition cond, gpointer data)
 	}
 
 	gstatus = g_io_channel_read_chars(io, buf, sizeof(buf) - 1,
-				&bytes_read, &gerr);
+							&bytes_read, &gerr);
 	if (gstatus != G_IO_STATUS_NORMAL) {
 		if (gerr)
 			g_error_free(gerr);
@@ -1115,8 +1110,8 @@ static gboolean sap_io_cb(GIOChannel *io, GIOCondition cond, gpointer data)
 		return TRUE;
 	}
 
-	if (handle_cmd(data, buf, bytes_read) < 0)
-		error("Invalid SAP message.");
+	if (handle_cmd(conn, buf, bytes_read) < 0)
+		error("SAP protocol processing failure.");
 
 	return TRUE;
 }
@@ -1132,17 +1127,18 @@ static void sap_io_destroy(void *data)
 
 		stop_guard_timer(conn);
 
-		if (conn->state != SAP_STATE_CONNECT_IN_PROGRESS)
+		if (conn->state != SAP_STATE_CONNECT_IN_PROGRESS &&
+				conn->state != SAP_STATE_CONNECT_MODEM_BUSY)
 			emit_property_changed(connection, server->path,
 					SAP_SERVER_INTERFACE, "Connected",
 					DBUS_TYPE_BOOLEAN, &connected);
 
 		if (conn->state == SAP_STATE_CONNECT_IN_PROGRESS ||
+				conn->state == SAP_STATE_CONNECT_MODEM_BUSY ||
 				conn->state == SAP_STATE_CONNECTED ||
 				conn->state == SAP_STATE_GRACEFUL_DISCONNECT)
 			sap_disconnect_req(NULL, 1);
 
-		conn->io = NULL;
 		sap_conn_remove(conn);
 	}
 }
@@ -1151,7 +1147,7 @@ static void sap_connect_cb(GIOChannel *io, GError *gerr, gpointer data)
 {
 	struct sap_connection *conn = data;
 
-	DBG("io %p gerr %p data %p ", io, gerr, data);
+	DBG("conn %p, io %p", conn, io);
 
 	if (!conn)
 		return;
@@ -1170,13 +1166,13 @@ static void connect_auth_cb(DBusError *derr, void *data)
 	struct sap_connection *conn = data;
 	GError *gerr = NULL;
 
-	DBG("derr %p data %p ", derr, data);
+	DBG("conn %p", conn);
 
 	if (!conn)
 		return;
 
 	if (derr && dbus_error_is_set(derr)) {
-		error("Access denied: %s", derr->message);
+		error("Access has been denied (%s)", derr->message);
 		sap_conn_remove(conn);
 		return;
 	}
@@ -1188,7 +1184,7 @@ static void connect_auth_cb(DBusError *derr, void *data)
 		return;
 	}
 
-	DBG("Client has been authorized.");
+	DBG("Access has been granted.");
 }
 
 static void connect_confirm_cb(GIOChannel *io, gpointer data)
@@ -1196,14 +1192,16 @@ static void connect_confirm_cb(GIOChannel *io, gpointer data)
 	struct sap_connection *conn = server->conn;
 	GError *gerr = NULL;
 	bdaddr_t src, dst;
+	char dstaddr[18];
 	int err;
 
-	DBG("io %p data %p ", io, data);
+	DBG("conn %p io %p", conn, io);
 
 	if (!io)
 		return;
 
 	if (conn) {
+		DBG("Another SAP connection already exists.");
 		g_io_channel_shutdown(io, TRUE, NULL);
 		return;
 	}
@@ -1233,23 +1231,24 @@ static void connect_confirm_cb(GIOChannel *io, gpointer data)
 		return;
 	}
 
-	err = btd_request_authorization(&src, &dst, SAP_UUID,
-					connect_auth_cb, conn);
+	ba2str(&dst, dstaddr);
+
+	err = btd_request_authorization(&src, &dst, SAP_UUID, connect_auth_cb,
+									conn);
 	if (err < 0) {
-		DBG("Authorization denied: %d %s", err,  strerror(err));
+		error("Authorization failure (err %d)", err);
 		sap_conn_remove(conn);
 		return;
 	}
 
-	DBG("SAP incoming connection (sock %d) authorization.",
-				g_io_channel_unix_get_fd(io));
+	DBG("Authorizing incomming SAP connection from %s", dstaddr);
 }
 
 static inline DBusMessage *message_failed(DBusMessage *msg,
 					const char *description)
 {
-	return g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
-				"%s", description);
+	return g_dbus_create_error(msg, ERROR_INTERFACE ".Failed", "%s",
+								description);
 }
 
 static DBusMessage *disconnect(DBusConnection *conn, DBusMessage *msg,
@@ -1257,8 +1256,6 @@ static DBusMessage *disconnect(DBusConnection *conn, DBusMessage *msg,
 {
 	struct sap_server *server = data;
 
-	DBG("server %p", server);
-
 	if (!server)
 		return message_failed(msg, "Server internal error.");
 
@@ -1269,7 +1266,7 @@ static DBusMessage *disconnect(DBusConnection *conn, DBusMessage *msg,
 
 	if (disconnect_req(server->conn, SAP_DISCONNECTION_TYPE_GRACEFUL) < 0)
 		return g_dbus_create_error(msg, ERROR_INTERFACE	".Failed",
-				"There is no active connection");
+					"There is no active connection");
 
 	return dbus_message_new_method_return(msg);
 }
@@ -1298,7 +1295,7 @@ static DBusMessage *get_properties(DBusConnection *c,
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
 	connected = (conn->state == SAP_STATE_CONNECTED ||
-			conn->state == SAP_STATE_GRACEFUL_DISCONNECT);
+				conn->state == SAP_STATE_GRACEFUL_DISCONNECT);
 	dict_append_entry(&dict, "Connected", DBUS_TYPE_BOOLEAN, &connected);
 
 	dbus_message_iter_close_container(&iter, &dict);
@@ -1332,8 +1329,8 @@ static void destroy_sap_interface(void *data)
 {
 	struct sap_server *server = data;
 
-	DBG("Unregistered interface %s on path %s",
-			SAP_SERVER_INTERFACE, server->path);
+	DBG("Unregistered interface %s on path %s", SAP_SERVER_INTERFACE,
+								server->path);
 
 	server_free(server);
 }
@@ -1391,10 +1388,10 @@ int sap_server_register(const char *path, bdaddr_t *src)
 	server->conn = NULL;
 
 	if (!g_dbus_register_interface(connection, path, SAP_SERVER_INTERFACE,
-				server_methods, server_signals, NULL,
-				server, destroy_sap_interface)) {
+					server_methods, server_signals, NULL,
+					server, destroy_sap_interface)) {
 		error("D-Bus failed to register %s interface",
-						SAP_SERVER_INTERFACE);
+							SAP_SERVER_INTERFACE);
 		goto server_err;
 	}
 
